
import java.io.*;
import java.sql.*;
import oracle.jdbc.driver.*;

/**
 * Test for "ORA-01460: unimplemented or unreasonable conversion requested"
 * by calling a stored procedure from JDBC when unicode conversion is involved.
 * <p>
 * Usage: java OraTestNclob filename
 * where filename is either small.txt, medium.txt or larger.txt (the 3 files
 * provided with the test case).
 * <p>
 * There error only occurs with medium.txt
 */
public class OraTestNclob {

    public static void createTable(Statement stmt) throws SQLException
    {
        ResultSet rs = stmt.executeQuery
            ("SELECT num_rows FROM user_tables WHERE table_name='NCLOBTEST'");
        try 
        {
            if (!rs.next())
            {
                // create the table
                stmt.execute("CREATE TABLE nclobtest (id NUMBER, sz INTEGER, text NCLOB)");

                // create the stored procedure for insertion
                stmt.execute("CREATE OR REPLACE PROCEDURE c_nclobtest (p_id NUMBER, p_size INTEGER, p_text CLOB CHARACTER SET ANY_CS) AS BEGIN INSERT INTO nclobtest VALUES (p_id, p_size, p_text); END;");
            }
        }
        finally
        {
            rs.close();
        }
    }

    public static void main(String[] args) throws Exception
    {
        String filename = args[0];

        Class.forName(oracle.jdbc.driver.OracleDriver.class.getName());
        String jdbcUrl = "jdbc:oracle:thin:@localhost:1521:orcl";
        Connection con =
            DriverManager.getConnection(jdbcUrl, "scott", "tiger");

        Statement stmt = null;
        CallableStatement pstmt = null;
        FileReader reader = null;
        try 
        {
            // create table and stored procedure if necessary
            stmt = con.createStatement();
            createTable(stmt);

            // insert text from the specified file
            File file = new File(filename);
            int size = (int)file.length();

            reader = new FileReader(file);

            pstmt = con.prepareCall("{call c_nclobtest(?,?,?)}");
            pstmt.setLong(1, System.currentTimeMillis());
            pstmt.setInt(2, size);
            ((OraclePreparedStatement)pstmt).setFormOfUse( 3, OraclePreparedStatement.FORM_NCHAR );
            pstmt.setCharacterStream(3, reader, size);
            pstmt.execute();
        }
        finally
        {
            if (reader != null)
            {
                try { reader.close(); }
                catch (Exception ignore) {}
            }
            if (stmt != null)
            {
                try { stmt.close(); }
                catch (Exception ignore) {}
            }
            if (pstmt != null)
            {
                try { pstmt.close(); }
                catch (Exception ignore) {}
            }

            con.close();
        }
    }
}
